/*  Program cel_nav-distance6.c  February 17, 2016
 *  This program calculates the arc distance from A to B on the surface
 *  of the earth.  Western location is A and eastern location is B.
 *  The spherical triangle is ABC with C being the north pole.
 *  Input is the latituce and longitude of A and B in decimal degrees.
 *  This version auto selects which geographical position (GP) is
 *  the east site or the west site.  Previous versions required entry
 *  of the western most site first.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

const double PI  =  3.14159265360;

FILE *outdata;

typedef struct input
{
double  AC1, wlonr, BC1, elonr;
} indata1;

  indata1 indata3;  /* This must be global for Borland C   */

/*** greeting ***********************************************************/
/* This function is the introduction the program cel_nav_distance.c     */

void greeting()
{
printf(
"\n"
"  Welcome to the program 'cel_nav_distance.c' by Joshua R. Mathison\n\n"
"  This program was written to get the distance in degrees of arc from\n"
"  point A and point B on the surface of the earth.\n\n"
"  Input is the latitude and longitude of each location in decimal degrees.\n\n"
"  Do not enter sites that are on the same meridian.\n\n"
"  South latitudes and East longitudes are negative.\n\n"
"  It does not matter which site you enter first.\n\n"
"  Output is the arc distance from A to B and the bearing of point A to B,\n"
"  and the bearing of B to A\n\n\n\n\n\n\n");
}
/************************************************************************/
/*** dd2r ***************************************************************/
/* This function receives an angle in decimal form and converts it      */
/* to radian measure.                                                   */

double dd2r(double decdeg)
{ double rad;
rad = ((decdeg * PI) / 180);
return rad;  }
/*** r2dd ***************************************************************/
/* This function receives an angle in radian form and returns the       */
/* angle in decimal form.                                               */

double r2dd(double raddeg)
{double decdeg;
decdeg = ((180 * raddeg) / PI);
return decdeg;  }
/************************************************************************/
/*** getinput ***********************************************************/

indata1 getinput()
{
double wlatdd, ac, wlondd, Wlonr,  elatdd, bc, elondd, Elonr,
tmpelatdd, tmpelondd, compwlondd, compelondd;
/*  compwlondd & compelondd are storage for minus longitude            */
/*  during compare below                                               */

int swap;  /*  boolian, 0 = false, 1 = true                        */

indata1 indata2;

/* This section gets the latitude of the first GP in decimal degrees  */
/* and returns co-latitude of the first GP in radians                 */

printf("  NOTE: keep six decimal places for best accuracy!\n\n");
printf("  Enter the latitude of the first GP in decimal degrees\n");
scanf("%lf", &wlatdd);
printf("  You entered %3.6lf\n", wlatdd);

/* This section gets the longitude of the first GP in decimal degrees */
/* and returns the longitude of the first GP in radians               */

printf("  Enter the longitude of the first GP in decimal degrees\n");
scanf("%lf", &wlondd);
printf("  You entered %3.6lf\n", wlondd);

/* This section gets the latitude of the second GP in decimal degrees  */
/* and returns co-latitude of the east site in radians                  */

printf("  Enter the latitude of the second GP in decimal degrees\n");
scanf("%lf", &elatdd);
printf("  You entered %3.6lf\n", elatdd);

/* This section gets the longitude of the second GP in decimal degrees */
/* and returns the longitude of the east site in radians                */

printf("  Enter the longitude of the second GP in decimal degrees\n");
scanf("%lf", &elondd);
printf("  You entered %3.6lf\n", elondd);


swap = 0;

if (wlondd < 0)
   { compwlondd = 360 + wlondd;  }
else { compwlondd = wlondd;  }
if (elondd < 0)
   { compelondd = 360 + elondd;  }
else { compelondd = elondd;  }

/* printf("compwlondd = %lf and compelondd = %lf\n", compwlondd ,compelondd);*/
/* fprintf(outdata,"compwlondd = %lf and compelondd = %lf\n", compwlondd ,compelondd); */

/*  if ((compwlondd > compelondd) && (compwlondd - compelondd > 180)) */
if ((compwlondd > compelondd)
   && (360 - compwlondd + compelondd < 180))
   { swap = 1;
     /* printf("1-swap = %d\n", swap); */
     /* fprintf(outdata, "1-swap = %d\n", swap); */
   }
else if ((compelondd > compwlondd) && (compelondd - compwlondd < 180))
   { swap = 1;
/* printf("2-swap = %d\n", swap); */
/* fprintf(outdata, "2-swap = %d\n", swap); */
   }

if (swap == 1)
   {
     tmpelatdd = elatdd;
     tmpelondd = elondd;
     elatdd    = wlatdd;
     elondd    = wlondd;
     wlatdd    = tmpelatdd;
     wlondd    = tmpelondd;
   }

/* printf("3-swap = %d\n", swap); */
/* fprintf(outdata, "3-swap = %d\n", swap); */

ac = dd2r(90 - wlatdd);       /* co-lat of the west site in radians */
Wlonr = dd2r(wlondd);
bc = dd2r(90 - elatdd);       /* co-lat of the east site in radians  */
Elonr = dd2r(elondd);

indata2.AC1 = ac;
indata2.wlonr = Wlonr;
indata2.BC1   = bc;
indata2.elonr = Elonr;

return indata2;

}

/************************************************************************/
/*** calcACB ************************************************************/
/* This function calculates angle ACB in radians for triangle one       */

double calcACB(double Wlonr, double Elonr)
{ double acb;
acb = Wlonr - Elonr;
return acb;  }
/************************************************************************/
/*** calcAB *************************************************************/
/* This function calculates the side AB of triangle.                    */

double calcAB(double ac, double bc, double acb)
{ double ab;
if (acb < 0.000001 && acb > -0.0000001)
{ ab = bc - ac; }
else {ab = acos((cos(bc) * cos(ac))+(sin(bc) * sin(ac)*cos(acb))); }
return ab;  }
/************************************************************************/
/*** calcABC ************************************************************/
/* This function calculates angle ABC in radians for triangle one       */
/* and converte to bearing in westerly direction.                       */

double calcABC(double ac, double bc, double ab, double acb)
{ double abc;
if (acb < 0.000001 && acb > -0.0000001)
{ abc = 0.000000; }
else {
abc = acos((cos(ac) - (cos(bc) * cos(ab))) / (sin(bc) * sin(ab)));
}
return (2 * PI) - abc; }

/************************************************************************/
/*** calcBAC ************************************************************/
/* This function calculates angle BAC in radians for triangle one       */

double calcBAC(double ac, double bc, double ab, double acb)
{ double bac;
if (acb < 0.000001 && acb > -0.0000001) { bac = 0.000000; }
else {
bac = acos((cos(bc) - (cos(ac) * cos(ab))) / (sin(ac) * sin(ab)));
}
return bac;  }

/************************************************************************/
/************************************************************************/
/************************************************************************/
/************************************************************************/

int main()
{

  double AC, wlonr, BC, elonr, BAC, ACB, ABC,
  AB;

  outdata = fopen("distance.txt", "a+");
  /* open or create a file for appending */

  if(outdata == NULL) {
    printf("Error: can't create file.\n");
    return 1;
  }
  else {
    printf("\n\n\n  Log file 'distance.txt' created successfully.\n");
    printf("  The output of this program is appended there.\n");
  }


/***  
  AC = getwlat();
  wlonr = getwlon();
  BC = getelat();
  elonr = getelon();
***/

  greeting();
  indata3 = getinput();   /*  new call to indata goes here */
  AC = indata3.AC1;
  wlonr = indata3.wlonr;
  BC = indata3.BC1;
  elonr = indata3.elonr;
  ACB  = calcACB(wlonr, elonr);
  AB   = calcAB(AC, BC, ACB);
  ABC  = calcABC(AC, BC, AB, ACB);
  BAC  = calcBAC(AC, BC, AB, ACB);

 printf("\n  Location A: %lf, %lf\n", (90 - r2dd(AC)), r2dd(wlonr));
 printf("  Location B: %lf, %lf\n", (90 - r2dd(BC)), r2dd(elonr));
 printf("  The distance A to B is %lf rad or %lf arc deg\n", AB, r2dd(AB));
 printf("  The bearing from B to A is %lf rad or %lf deg\n", ABC, r2dd(ABC));
 printf("  The bearing from A to B is %lf rad or %lf deg\n\n", BAC, r2dd(BAC));

fprintf(outdata, "\n  Location A: %lf, %lf\n", (90 - r2dd(AC)), r2dd(wlonr));
fprintf(outdata, "  Location B: %lf, %lf\n", (90 - r2dd(BC)), r2dd(elonr));
fprintf(outdata, "  The distance A to B is %lf rad or %lf arc deg\n"
, AB, r2dd(AB));
fprintf(outdata, "  The bearing from B to A\
 is %lf rad or %lf deg\n" , ABC, r2dd(ABC));
fprintf(outdata, "  The bearing from A to B\
 is %lf rad or %lf deg\n\n\n" , BAC, r2dd(BAC));

return 0;
}   /* END MAIN */

