



/***
Program cel_nav-run_lop9.c
This program written by Joshua R. Mathison.
Enter a geographical position (GP) in latitude and longitude in decimal
degrees, a bearing from true north from there in decimal degrees, and
a distance in decimal degrees of arc from the above GP.
The program returns a new GP which is the bearing from, and distance
from the original (entered) GP.
***/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

  FILE *outdata;  /* This must be global for fprt2dms print to file     */

const double PI  =  3.14159265360;

/*** greeting ***********************************************************/
/* This function is the introduction the program cel_nav-run_lop.c */

void greeting()
{
printf(
"   Program cel_nav-run_lop.c\n\n"
"   This program written by Joshua R. Mathison\n\n"
"   This program was written to run a new line of postition\n"
"   by running the Geographical Position ahead or behind.\n\n"
"   Enter a geographical position (GP) in latitude and\n"
"   longitude in decimal degrees, a bearing from true north from\n"
"   there in decimal degrees, and a distance in decimal degrees\n"
"   of arc from the above GP.\n\n"
"   Do NOT enter a bearing of 0 or 180 degrees.  Bearing must be\n"
"   plus or minus at least 0.1 degree.\n\n"
"   The program returns a new GP which is the bearing from,\n"
"   and distance from the original (entered) GP.\n\n"
"            Press return to continue.\n\n\n\n\n\n");
}
/************************************************************************/
/*** dms2dd *************************************************************/
/* This function receives degrees, minutes, and seconds and returns     */
/* the angle as degrees plus a decimal.                                 */

double dms2dd(int deg, int min, int sec)
{ double decdeg;
decdeg = (deg + (((60 * min) + sec) / 3600));
return decdeg;
}
/*** dd2r ***************************************************************/
/* This function receives an angle in decimal form and converts it      */
/* to radian measure.                                                   */

double dd2r(double decdeg)
{ double rad; rad = ((decdeg * PI) / 180);
return rad;
}
/*** r2dd ***************************************************************/
/* This function receives an angle in radian form and returns the       */
/* angle in decimal form.                                               */

double r2dd(double raddeg)
{double decdeg; decdeg = ((180 * raddeg) / PI);
return decdeg;
}
/************************************************************************/
/*** getlat  ************************************************************/
/* This function gets the latitude of the GP co-ord in decimal degrees  */
/* and returns co-latitude of the GP co-ord in radians                  */

double getlat()
{ double latdd, gpcolat;
printf("  NOTE: do NOT use NUM-PAD for entering data!\n\n");
printf("  NOTE: keep six decimal places for best accuracy!\n\n");
printf("  Enter the latitude of the GP co-ord in decimal degrees\n");
scanf("%lf", &latdd);
printf("\n  You entered %3.6lf\n", latdd);
gpcolat = dd2r(90 - latdd);        /* co-lat of the GP in radians       */
return gpcolat;
}
/************************************************************************/
/*** getlon  ************************************************************/
/* This function gets the longitude of the GP co-ord in decimal degrees */
/* and returns the longitude of the GP co-ord in radians                */

double getlon()
{ double gplondd, gplonr;
printf("  Enter the longitude of the GP co-ord in decimal degrees\n");
scanf("%lf", &gplondd);
printf("\n  You entered %3.6lf\n", gplondd);
gplonr = dd2r(gplondd);
return gplonr;
}
/************************************************************************/
/*** getbearing  ********************************************************/
/* This function gets the bearing from the GP towards its new location **/

double getbearing()
{ double bearingdd, bearingr;
printf("  Enter the bearing from the GP towards its new location\n");
printf("  in decimal degrees\n");
scanf("%lf", &bearingdd);
printf("\n  You entered %3.6lf\n", bearingdd);
bearingr = dd2r(bearingdd);
return bearingr;
}
/************************************************************************/
/*** getdistance  *******************************************************/
/* This function gets the distance in decimal degrees from the GP       */
/* towards its new location - one min of arc per nautical mile          */

double getdistance()

{ double distancedd, distancer;
printf("  Enter the distance in decimal degrees from the GP towards its\n");
printf("  new location - one min of arc per nautical mile\n");
scanf("%lf", &distancedd);
printf("\n  You entered %3.6lf\n", distancedd);
distancer = dd2r(distancedd);
return distancer;
}
/************************************************************************/
/*** calcAC  ************************************************************/
/* This function calculates side AC for the triangle                    */
/* use for bearings from 180 to 360 degrees                               */

double calcAC(double ab, double bc, double abc)

{ double ac;
ac = acos((cos(ab) * cos(bc)) + (sin(ab) * sin(bc) * cos(abc)));
return ac;
}
/************************************************************************/
/*** calcBC  ************************************************************/
/* This function calculates side BC for the triangle                    */
/* use for bearings from 0 to 180 degrees                               */

double calcBC(double ac, double ab, double cab)

{ double bc;
bc = acos((cos(ac) * cos(ab)) + (sin(ac) * sin(ab) * cos(cab)));
return bc;
}
/************************************************************************/
/*** calcACB ************************************************************/
/* This function calculates angle ACB for the triangle                  */

double calcACB(double ab, double bc, double ac)

{ double acb;
acb =acos((cos(ab) - (cos(bc) * cos(ac))) / (sin(bc) * sin(ac)));
return acb;
}
/************************************************************************/
/*** prtr2dms ***********************************************************/
/* This function receives an angle in radians and prints it in DMS      */

void prtr2dms(double degr)
{
int deg, min, sec;  double dmin, decd, decdtmp;
decd = r2dd(degr);
if (decd < 0) {
decdtmp  =  -decd;
deg = floor(decdtmp); /* deg = truncf(decd); trunc NOT work */
dmin = ((decdtmp - floor(decdtmp)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
printf("-%d %d %d ", deg, min, sec);
}
else {
deg = floor(decd); /* deg = truncf(decd); trunc NOT work */
dmin = ((decd - floor(decd)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
/* else if (sec <= -60) { min = min - 1; sec = 0; }      */
/* else if (min >= 60) { deg = deg + 1; min = 0; }       */
/* else if (min <= -60) { deg = deg -1; min = 0; }       */
printf("%d %d %d ", deg, min, sec);
}
}
/************************************************************************/
/*** fprtr2dms **********************************************************/
/* This function receives an angle in radians and prints it in DMS      */
/* to file outdata                                                      */

void fprtr2dms(double degr)
{
int deg, min, sec;  double dmin, decd, decdtmp;
decd = r2dd(degr);
if (decd < 0) {
decdtmp  =  -decd;
deg = floor(decdtmp); /* deg = truncf(decd); trunc NOT work */
dmin = ((decdtmp - floor(decdtmp)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
fprintf(outdata, "-%d %d %d ", deg, min, sec);
}
else {
deg = floor(decd); /* deg = truncf(decd); trunc NOT work */
dmin = ((decd - floor(decd)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
/* else if (sec <= -60) { min = min - 1; sec = 0; }      */
/* else if (min >= 60) { deg = deg + 1; min = 0; }       */
/* else if (min <= -60) { deg = deg -1; min = 0; }       */
fprintf(outdata, "%d %d %d ", deg, min, sec);
}
}
/************************************************************************/
/************************************************************************/
int main()
{
/*  FILE *outdata;  Must be global for fprt2dms print to file           */
  int    move_on;
  double GPCOLAT, GPLON, BEARING, DISTANCE, NEWCOLAT,
         NEWLON, ACB, BEARING2;  /*  all in radian measure */

  outdata = fopen("run-lop.txt", "a+");
  /* open or create a file for appending */

  if(outdata == NULL) {
    printf("Error: can't create file.\n");
    return 1;
  }
  else {
    printf("\n\n   Log file 'run-lop.txt' created successfully.\n");
    printf("   The output of this program will be appended there.\n\n");
  }

    greeting();
       move_on = getchar();
    GPCOLAT    =   getlat();
    GPLON      =   getlon();
    BEARING    =   getbearing();
    DISTANCE   =   getdistance();

  if (BEARING  >= PI)
     {
       BEARING2   = ((2 * PI) - BEARING);
       NEWCOLAT   =   calcAC(DISTANCE, GPCOLAT, BEARING2);
                  /*  calcAC(double ab, double bc, double abc) */
       ACB       =    calcACB(DISTANCE, GPCOLAT, NEWCOLAT);
                   /* calcACB(double ab, double bc, double ac) */
       NEWLON   =   GPLON  + ACB;
     }
  else
     {
       NEWCOLAT   =   calcBC(GPCOLAT, DISTANCE, BEARING);
                   /* calcBC(double ac, double ab, double cab) */
       ACB       =    calcACB(DISTANCE, NEWCOLAT, GPCOLAT);
                   /* calcACB(double ab, double bc, double ac) */
       NEWLON   =   GPLON - ACB;
     }

  if (NEWLON > PI)
     { NEWLON = ((2 * -PI) + NEWLON); }
  else if (NEWLON < -PI)
     { NEWLON = ((2 * PI) + NEWLON); }


printf("\n\n    The original location was at \n");
printf("    latitude: %lf and longitude: %lf\n"
                         , r2dd((PI / 2) - GPCOLAT), r2dd(GPLON));
printf("    and in DMS format "); prtr2dms((PI / 2) - GPCOLAT);
printf(" and "); prtr2dms(GPLON); printf("\n");
printf("\n    The new location is %lf", r2dd(DISTANCE));
printf(" arc degrees from there\n");
printf("    at a bearing from true north of %lf degrees" , r2dd(BEARING));
printf("\n\n    The new co-ords in decimal degrees are\n");
printf("    latitude: %lf and longitude: %lf\n"
                               , r2dd((PI / 2) - NEWCOLAT), r2dd(NEWLON));
printf("    and in DMS format ");  prtr2dms((PI / 2) - NEWCOLAT);
printf(" and "); prtr2dms(NEWLON); printf("\n\n");

fprintf(outdata, "\n\n    The original location was at \n");
fprintf(outdata, "    latitude: %lf and longitude: %lf\n"
                                  , r2dd((PI / 2) - GPCOLAT), r2dd(GPLON));
fprintf(outdata, "    and in DMS format "); fprtr2dms((PI / 2) - GPCOLAT);
fprintf(outdata, " and "); fprtr2dms(GPLON);
fprintf(outdata, "\n");
fprintf(outdata, "\n    The new location is %lf", r2dd(DISTANCE));
fprintf(outdata, " arc degrees from there\n");
fprintf(outdata, "    at a bearing from true north of %lf degrees" , r2dd(BEARING));
fprintf(outdata, "\n\n    The new co-ords in decimal degrees are\n");
fprintf(outdata, "    latitude: %lf and longitude: %lf\n"
                                  , r2dd((PI / 2) - NEWCOLAT), r2dd(NEWLON));
fprintf(outdata, "    and in DMS format "); fprtr2dms((PI / 2) - NEWCOLAT);
fprintf(outdata, " and "); fprtr2dms(NEWLON);
fprintf(outdata, "\n\n");

return 0;
}   /* END MAIN */

