



/***
   Program cel_nav-position5.c  written by Joshua R. Mathison.
   February 17, 2016

   This program calculates the location in lat and long
   of a position on a map from lat and long of two known locations
   and the arc distance in degrees from each location to the position.
   There are a total of five spherical triangles involved.
   The first is location A and B and C, the north pole.  The
   second triangle is location A and B and the position north of line AB.
   The fourth triangle is the image of the second triangle with line
   AB and the position south of line AB.  The third and fifth triangles
   yeild the lat and long of the position at its potential position
   north of AB and south of AB.
   The position is called spot in the code below.
***/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

FILE *outdata;
const double PI  =  3.14159265360;

typedef struct input
{
double  AC1, wlonr, BC1, elonr, AC2, BC2;
} indata1;

  indata1 indata3;  /* This must be global for Borland C   */

/*** greeting1***********************************************************/
/* This function is the introduction the program cel_nav-position.c     */

void greeting1()
{
printf(
"\n\n"
"  Welcome to the program 'cel_nav-position.c.'\n"
"  Written by Joshua R. Mathison\n\n"
"  This program was written to directly compute, without using charts,\n"
"  an observers position on earth using the geographic position of two\n"
"  or more heavenly bodies and the corrected sextant co-altitude to each.\n"
"\n"
"  One only needs to enter into the program the co-ordinates of two known\n"
"  geographic positions (GP) and the arc distances (90 degrees minus\n"
"  corrected sextant altitude) to each geographic position.\n"
"  All input is to be in decimal degrees! North latitude and west\n"
"  longitude are positive, south lat and east long are negative.\n\n"

"  Press return to continue.\n\n");
}

/************************************************************************/
/*** greeting2***********************************************************/
/* This function is the introduction the program cel_nav-position.c     */

void greeting2()
{
printf(
"\n\n\n\n"
"  The program computations are based on the fact that your line of\n"
"  position from each geographic position is two intersecting circles\n"
"  centered on each geographic position and with a radius of 90 degrees\n"
"  minus the corrected sextant altitude to each. (co-altitude)\n"
"  \n"
"  This program prints the position of the two intersections, but you\n"
"  are only positioned on ONE of them.\n"
"  \n"
"  If it is not obvious which intersection you are on, get the sextant\n"
"  altitude to a third heavenly body and run this program again with the\n"
"  third GP and co-altitude and one of the first two GP's and\n"
"  co-altitudes.\n\n"
"  Press return to continue.\n\n");
}

/************************************************************************/
/*** disclaimer *********************************************************/
/* This function is the disclaimer to the program cel_nav-position.c    */

void disclaimer()
{
printf(
"\n\n\n\n"
"  Use this program at your own risk!  It has not been tested under\n"
"  actual real life navigational conditions.\n\n"
"  Do not enter into this program the geographical position of two\n"
"  heavenly bodies if their bearings from you are 180 degrees apart\n"
"  as this program may fail.  Also do not enter two GP on the same\n"
"  meridan.\n\n"
"  These two bugs are the only ones revealed by extensive testing\n"
"  using simulated GP.\n\n"
"  Press return to continue.\n\n\n\n");
}

/************************************************************************/
/*** dms2dd *************************************************************/
/* This function receives degrees, minutes, and seconds and returns     */
/* the angle as degrees plus a decimal.                                 */

double dms2dd(int deg, int min, int sec)
{ double decdeg;
decdeg = (deg + (((60 * min) + sec) / 3600));
return decdeg;  }

/*** dd2r ***************************************************************/
/* This function receives an angle in decimal form and converts it      */
/* to radian measure.                                                   */

double dd2r(double decdeg)
{ double rad;
rad = ((decdeg * PI) / 180);
return rad;  }

/*** r2dd ***************************************************************/
/* This function receives an angle in radian form and returns the       */
/* angle in decimal form.                                               */

double r2dd(double raddeg)
{double decdeg;
decdeg = ((180 * raddeg) / PI);
return decdeg; }

/************************************************************************/
/*** getinput ***********************************************************/

indata1 getinput()
{ 
double wlatdd, ac, wlondd, Wlonr, wmiles, ac2, elatdd, bc, elondd, Elonr,
emiles, bc2, tmpelatdd, tmpelondd, tmpemiles, compwlondd, compelondd;
/*  compwlondd & compelondd are storage for minus longitude            */
/*  during compare below                                               */

int swap;  /*  boolian, 0 = false, 1 = true                        */

indata1 indata2;

/* This section gets the latitude of the first GP in decimal degrees  */
/* and returns co-latitude of the first GP in radians                 */

printf("  NOTE: keep six decimal places for best accuracy!\n\n");
printf("  Enter the latitude of the first GP in decimal degrees\n");
scanf("%lf", &wlatdd);
printf("  You entered %3.6lf\n", wlatdd);

/* This section gets the longitude of the first GP in decimal degrees */
/* and returns the longitude of the first GP in radians               */

printf("  Enter the longitude of the first GP in decimal degrees\n");
scanf("%lf", &wlondd);
printf("  You entered %3.6lf\n", wlondd);

/* This section gets the co-altitude (90 - hc) to first GP  */
/* spot                                                     */

printf("  Enter the co-altitude (90 - hc) to first GP\n");
scanf("%lf",&wmiles);
printf("  You entered %lf degrees.\n\n", wmiles);

/* This section gets the latitude of the second GP in decimal degrees  */
/* and returns co-latitude of the east site in radians                  */

printf("  Enter the latitude of the second GP in decimal degrees\n");
scanf("%lf", &elatdd);
printf("  You entered %3.6lf\n", elatdd);

/* This section gets the longitude of the second GP in decimal degrees */
/* and returns the longitude of the east site in radians                */

printf("  Enter the longitude of the second GP in decimal degrees\n");
scanf("%lf", &elondd);
printf("  You entered %3.6lf\n", elondd);


/* This section gets the co-altitude (90 - hc) to second GP     */
/* spot                                                         */

printf("  Enter the co-altitude (90 - hc) to second GP\n");
scanf("%lf",&emiles);
printf("  You entered %lf degrees.\n\n", emiles);

swap = 0;

if (wlondd < 0)
   { compwlondd = 360 + wlondd;  }
else { compwlondd = wlondd;  }
if (elondd < 0)
   { compelondd = 360 + elondd;  }
else { compelondd = elondd;  }

/* printf("compwlondd = %lf and compelondd = %lf\n", compwlondd ,compelondd); */
/* fprintf(outdata,"compwlondd = %lf and compelondd = %lf\n", compwlondd ,compelondd); */

/*  if ((compwlondd > compelondd) && (compwlondd - compelondd > 180)) */
if ((compwlondd > compelondd)
   && (360 - compwlondd + compelondd < 180))
   { swap = 1;
/*     printf("1-swap = %d\n", swap); */
/*     fprintf(outdata, "1-swap = %d\n", swap); */
   }
else if ((compelondd > compwlondd) && (compelondd - compwlondd < 180))
   { swap = 1;
/* printf("2-swap = %d\n", swap); */
/* fprintf(outdata, "2-swap = %d\n", swap); */
   }

if (swap == 1)
   {
     tmpelatdd = elatdd;
     tmpelondd = elondd;
     tmpemiles = emiles;
     elatdd    = wlatdd;
     elondd    = wlondd;
     emiles    = wmiles;
     wlatdd    = tmpelatdd;
     wlondd    = tmpelondd;
     wmiles    = tmpemiles;
   }

/*  printf("3-swap = %d\n", swap); */
/* fprintf(outdata, "3-swap = %d\n", swap); */

/***
ac = dd2r(90 - wlatdd);         co-lat of the west site in radians
return ac;
return Wlonr = dd2r(wlondd);
return ac2 = dd2r(wmiles);
bc = dd2r(90 - elatdd);         co-lat of the east site in radians
return bc;
return Elonr = dd2r(elondd);
return bc2 = dd2r(emiles);
double  AC1, wlonr, BC1, elonr, AC2, BC2;
***/

ac = dd2r(90 - wlatdd);       /* co-lat of the west site in radians */
Wlonr = dd2r(wlondd);
ac2 = dd2r(wmiles);
bc = dd2r(90 - elatdd);       /* co-lat of the east site in radians  */
Elonr = dd2r(elondd);
bc2 = dd2r(emiles);

indata2.AC1 = ac;
indata2.wlonr = Wlonr;
indata2.BC1   = bc;
indata2.elonr = Elonr;
indata2.AC2   = ac2;
indata2.BC2   = bc2;

return indata2;

}

/************************************************************************/
/*** calcACB1 ***********************************************************/
/* This function calculates angle ACB in radians for triangle one       */

double calcACB1(double Wlonr, double Elonr)
{ double acb1;
acb1 = Wlonr - Elonr;
return acb1;  }

/*** calcAB12 ************************************************************/
/* This function calculates the side AB1 of triangle one which           */
/* equals AB2 in triangle two                                            */

double calcAB12(double ac1, double bc1, double acb1)
{ double ab1;
if (acb1 < 0.000001 && acb1 > -0.0000001) { ab1 = bc1 - ac1; }
else {ab1 = acos((cos(bc1) * cos(ac1))+(sin(bc1) * sin(ac1)*cos(acb1)));}
return ab1;
}

/*** calcABC1 ***********************************************************/
/* This function calculates angle ABC in radians for triangle one       */

double calcABC1(double ac1, double bc1, double ab1, double acb1)
{ double abc1;
if (acb1 < 0.000001 && acb1 > -0.0000001) { abc1 = 0.000000; }
else {
abc1 = acos((cos(ac1) - (cos(bc1) * cos(ab1))) / (sin(bc1) * sin(ab1)));
}
return abc1;
}

/*** calcABC2 ***********************************************************/
/* This function calculates angle ABC2 for triangle two                 */

double calcABC2(double ac2, double bc2, double ab1)
{ double abc2;
abc2 = acos((cos(ac2) - (cos(bc2) * cos(ab1))) / (sin(bc2) * sin(ab1)));
return abc2;
}

/*** calcABC3 ***********************************************************/
/* This function calculates angle ABC3 for triangle three               */

double calcABC3(double abc1, double abc2)
{ double abc3;
abc3 = abc1 - abc2; if (abc2 > abc1) { abc3 = abc2 - abc1; }
return abc3;
}

/*** calcAC3 ************************************************************/
/* This function calculates side AC3 for triangle three                 */
/* which is the co-latitude of the spot                                 */

double calcAC3(double bc2, double bc1, double abc3)
{ double ac3;
ac3 = acos((cos(bc2) * cos(bc1)) + (sin(bc1) * sin(bc2) * cos(abc3)));
return ac3;
}

/*** calcACB3 ***********************************************************/
/* This function calculates angle ACB3 for triangle three               */
/* which plus or minus elonr is the longitude of the spot               */

double calcACB3(double bc2, double bc1, double ac3)

{ double acb3;
acb3 =acos((cos(bc2) - (cos(bc1) * cos(ac3))) / (sin(bc1) * sin(ac3)));
return acb3;
}

/*** calcSPOTlat ********************************************************/
/* This function calculates the latitude of the spot on the map from    */
/* the co-latitude given by side AB3 of triangle three                  */

double calcSPOTlat(double ac3)
{ double SPOTlat;
SPOTlat = ((PI / 2) - ac3);
return SPOTlat;  }

/*** calcSPOTlon1********************************************************/
/* This function calculates the longitude of the spot1 on the map which */
/* is the sum of elonr plus or minus angle ACB3                         */

double calcSPOTlon1(double Elonr, double acb3, double abc1, double abc2)
{ double SPOTlon1; SPOTlon1 = Elonr + acb3;
if (abc2 > abc1) { SPOTlon1 = Elonr - acb3; }
return SPOTlon1;
}

/*** calcSPOTlon2********************************************************/
/* This function calculates the longitude of the spot2 on the map which */
/* is the sum of elonr plus or minus angle ACB5                         */

double calcSPOTlon2(double Elonr, double acb5, double abc1, double abc2)
{ double SPOTlon2; SPOTlon2 = Elonr + acb5;
if (abc1 + abc2 > PI) { SPOTlon2 = Elonr - acb5; }
return SPOTlon2;
}

/************************************************************************/
/*** prtr2dms ***********************************************************/
/* This function receives an angle in radians and prints it in DMS      */

void prtr2dms(double degr)
{
int deg, min, sec;  double dmin, decd, decdtmp;
decd = r2dd(degr);
if (decd < 0) {
decdtmp  =  -decd;
deg = floor(decdtmp); /* deg = truncf(decd); trunc NOT work */
dmin = ((decdtmp - floor(decdtmp)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
printf("-%d %d %d ", deg, min, sec);
}
else {
deg = floor(decd); /* deg = truncf(decd); trunc NOT work */
dmin = ((decd - floor(decd)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
/* else if (sec <= -60) { min = min - 1; sec = 0; }      */
/* else if (min >= 60) { deg = deg + 1; min = 0; }       */
/* else if (min <= -60) { deg = deg -1; min = 0; }       */
printf("%d %d %d ", deg, min, sec);
}
}

/************************************************************************/
/*** fprtr2dms **********************************************************/
/* This function receives an angle in radians and prints it in DMS      */
/* to file outdata                                                      */

void fprtr2dms(double degr)
{
int deg, min, sec;  double dmin, decd, decdtmp;
decd = r2dd(degr);
if (decd < 0) {
decdtmp  =  -decd;
deg = floor(decdtmp); /* deg = truncf(decd); trunc NOT work */
dmin = ((decdtmp - floor(decdtmp)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
fprintf(outdata, "-%d %d %d ", deg, min, sec);
}
else {
deg = floor(decd); /* deg = truncf(decd); trunc NOT work */
dmin = ((decd - floor(decd)) * 60);
min = floor(dmin); sec = ((dmin - min) * 60);
if (sec >= 60) { min = min + 1; sec = 0; }
/* else if (sec <= -60) { min = min - 1; sec = 0; }      */
/* else if (min >= 60) { deg = deg + 1; min = 0; }       */
/* else if (min <= -60) { deg = deg -1; min = 0; }       */
fprintf(outdata, "%d %d %d ", deg, min, sec);
}
}
/************************************************************************/
/************************************************************************/
/************************************************************************/
int main()
{

/*  FILE *outdata; -- moved to global                                     */
  int  move_on;
  double AC1, wlonr, AC2, BC1, elonr, Eradius, BC2, ACB1, ABC1,
  AB12, ABC2, ABC3, ABC5, AC3, ACB3, ACB5, spotlat, spotlon1, spotlon2;

  outdata = fopen("position.txt", "a+");
  /* open or create a file for appending */

  if(outdata == NULL) {
    printf("Error: can't create file.\n");
    return 1;
  }
  else {
    printf("\n  Log file 'position.txt' created successfully.\n");
    printf("  The output of this program is appended there.\n");
  }


/* double  AC1, wlonr, BC1, elonr, AC2, BC2; */

  greeting1();
  move_on = getchar();
  greeting2();
  move_on = getchar();
  disclaimer();
  move_on = getchar();
  indata3 = getinput();   /*  new call to indata goes here */
  AC1 = indata3.AC1;
  wlonr = indata3.wlonr;
  BC1 = indata3.BC1;
  elonr = indata3.elonr;
  ACB1 = calcACB1(wlonr, elonr);
  AB12 = calcAB12(AC1, BC1, ACB1);

/***
  printf("  The side AB12 is %lf rad or %lf deg\n\n", AB12, r2dd(AB12));
***/

  AC2 = indata3.AC2;
  BC2 = indata3.BC2;
  ABC1 = calcABC1(AC1, BC1, AB12, ACB1);
  ABC2 = calcABC2(AC2, BC2, AB12);

  ABC3 = calcABC3(ABC1,  ABC2);
  AC3 = calcAC3(BC2, BC1, ABC3);
  ACB3 = calcACB3(BC2, BC1, AC3);
  spotlat = calcSPOTlat(AC3);
  spotlon1 = calcSPOTlon1(elonr, ACB3, ABC1, ABC2);

  if (spotlon1 > PI)
     { spotlon1 = ((2 * -PI) + spotlon1); }
  else if (spotlon1 < -PI)
     { spotlon1 = ((2 * PI) + spotlon1); }

/***
  printf("  Co-lat of side AC1 is %lf rad or %lf deg\n", AC1, r2dd(AC1));
  printf("  Lon of West site is %lf rad or %lf deg\n", wlonr, r2dd(wlonr));
  printf("  Co-lat of side BC1 is %lf rad or %lf deg\n", BC1, r2dd(BC1));
  printf("  Lon of East site is %lf rad or %lf deg\n", elonr, r2dd(elonr));
  printf("  Dis from W site to spot: %lf rad or %lf deg \n", AC2, r2dd(AC2));
  printf("  Dis from E site to spot: %lf rad or %lf deg\n", BC2, r2dd(BC2));
  printf("  The angle ACB1 at pole is %lf rad or %lf deg\n", ACB1, r2dd(ACB1));
  printf("  The angle ABC1 is %lf radians or %lf degrees\n", ABC1, r2dd(ABC1));
  printf("  The angle ABC2 is %lf rad or %lf degrees\n\n", ABC2, r2dd(ABC2));

  printf("  The angle ABC3 is %lf rad or %lf degrees\n", ABC3, r2dd(ABC3));
  printf("  The co-lat of spot is %lf rad or %lf deg\n", AC3, r2dd(AC3));
  printf("  The angle ACB3 is %lf rad or %lf degrees\n", ACB3, r2dd(ACB3));
***/

printf("  :) Position 1-?? co-ords are latitude: %lf and longitude %lf\n", 
r2dd(spotlat), r2dd(spotlon1));
printf("  and in DMS format latitude: "); prtr2dms(spotlat);
printf(" and longitude "); prtr2dms(spotlon1); printf("\n\n");

fprintf(outdata, "  :) Position 1-?? co-ords are latitude: %lf and long\
itude %lf\n", r2dd(spotlat), r2dd(spotlon1));
fprintf(outdata, "  and in DMS format latitude: "); fprtr2dms(spotlat);
fprintf(outdata, " and longitude "); fprtr2dms(spotlon1);
fprintf(outdata, "\n\n");

  ABC5 = ABC1 + ABC2;
  AC3 = calcAC3(BC2, BC1, ABC5);
  ACB5 = calcACB3(BC2, BC1, AC3);
  spotlat = calcSPOTlat(AC3);
  spotlon2 = calcSPOTlon2(elonr, ACB5, ABC1, ABC2);

  if (spotlon2 > PI)
     { spotlon2 = ((2 * -PI) + spotlon2); }
  else if (spotlon2 < -PI)
     { spotlon2 = ((2 * PI) + spotlon2); }

/***
  printf("  The angle ABC5 is %lf rad or %lf degrees\n", ABC3, r2dd(ABC5));
  printf("  The co-lat of spot is %lf rad or %lf deg\n", AC3, r2dd(AC3));
  printf("  The angle ACB5 is %lf rad or %lf degrees\n", ACB3, r2dd(ACB5));
***/

printf("  :) Position 2-?? co-ords are latitude: %lf and longitude: %lf\n", 
r2dd(spotlat), r2dd(spotlon2));
printf("  and in DMS format latitude: "); prtr2dms(spotlat);
printf(" and longitude: "); prtr2dms(spotlon2); printf("\n");

fprintf(outdata, "  :) Position 2-?? co-ords are latitude: %lf and longitude: %lf\n",
r2dd(spotlat), r2dd(spotlon2));
fprintf(outdata, "  and in DMS format latitude: "); fprtr2dms(spotlat);
fprintf(outdata, " and longitude: "); fprtr2dms(spotlon2);
fprintf(outdata, "\n\n\n");

return 0;
}   /* END MAIN */

